﻿using System;
using System.Collections.Generic;
using System.Linq;
using AutoMapper;
using Core.Exceptions;
using System.Dynamic;

namespace Data.Core.Model
{
    public class Candidate
    {
        

        #region Boring data code
        internal void Delete()
        {
            if (!this.ID.HasValue)
            {
                throw new InvalidPropertyException("Need an ID to delete", "ID");
            }

            // no need to check for ORM, GetCandidate will do that
            var candidate = GetCandidate(this.ID.Value);
            this.ORM.database.Candidates.DeleteOnSubmit(candidate);
        }

        public Candidate(DataStorage dataStorage, int id)
            : this(dataStorage)
        {
            this.GetCandidate(id);
        }

        public Candidate(DataStorage dataStorage, dynamic filters)
            : this(dataStorage)
        {
            this.Load(filters);
        }

        private DataAccess.Candidate GetCandidate(int Id)
        {
            dynamic filters = new ExpandoObject();
            filters.Id = Id;
            return GetCandidate(filters);
        }

        private DataAccess.Candidate GetCandidate(dynamic filters)
        {
            if (this.ORM == null)
            {
                throw new InvalidPropertyException("ORM has not been set", "ORM");
            }

            var objectTester = new ObjectTester<DataAccess.Candidate>((IDictionary<string, object>)filters);
            return this.ORM.database.Candidates.ToList().FirstOrDefault(_ => objectTester.Match(_));
        }

        private void Load(dynamic filters)
        {
            var candidate = GetCandidate(filters);
            if (candidate != null)
            {
                Mapper.Map(candidate, this);
            }
        }
        #endregion      
    }
}
