﻿using System;
using Data.Core.DataAccess;
using AutoMapper;
using System.Data.Linq;
using System.Collections.Generic;
using System.Linq;

namespace Data.Core.Model
{
    public class DataStorage : IDisposable
    {
        internal CVDataContext database = new CVDataContext();

        public DataStorage()
        {
            #region Configure Automapper - not important for what we doing
            Mapper.CreateMap<Binary, byte[]>().ConvertUsing(_ =>
                    {
                        return _.ToArray();
                    });

            Mapper.CreateMap<byte[], Binary>().ConvertUsing(_ =>
                {
                    return new Binary(_);
                });

            Mapper.CreateMap<string, List<string>>().ConvertUsing(_ =>
                {
                    if (!string.IsNullOrWhiteSpace(_))
                    {
                        return new List<string>(_.Split(','));
                    }
                    else
                    {
                        return new List<string>();
                    }
                });

            Mapper.CreateMap<List<string>, string>().ConvertUsing(_ =>
                {
                    if (_.Any())
                    {
                        return _.Aggregate((set, next) =>
                            {
                                return set += string.IsNullOrWhiteSpace(set) ? next : string.Format(",{0}", next);
                            });
                    }
                    else
                    {
                        return null;
                    }
                });

            Mapper.CreateMap<DataAccess.Candidate, Model.Candidate>()
                .ForAllMembers(options => options.Condition(_ => !_.IsSourceValueNull));

            Mapper.CreateMap<Model.Candidate, DataAccess.Candidate>()
                .ForAllMembers(options => options.Condition(_ => !_.IsSourceValueNull)); 
            #endregion
        }

        public void Commit()
        {
            database.SubmitChanges();
        }

        public void Dispose()
        {
            database.Dispose();
        }
    }
}
