﻿using System;
using System.Linq;
using ZeroMQ;
using Core.Exceptions;
using System.Collections.Concurrent;
using System.ServiceModel;
using System.Collections.Generic;
using System.Threading;

namespace Queue
{
    public class QueueAccess : IDisposable, IQueueAccess
    {
        readonly ZmqSubscribeSocket subscriber;
        private readonly AccessType accessType;
        private readonly string host;
        private readonly List<string> queue;

        public QueueAccess() : this("tcp://127.0.0.1:50000", AccessType.Sender) { }

        public QueueAccess(string host, AccessType accessType)
        {
            this.accessType = accessType;
            this.host = host;
            if (accessType == AccessType.Receiver)
            {
                queue = new List<string>();
                subscriber = new ZmqSubscribeSocket();
                subscriber.Connect(host);
                subscriber.Subscribe(string.Empty);                
            }
        }

        public void Send(string message)
        {
            if (this.accessType != AccessType.Sender)
            {
                throw new InvalidPropertyException("Queue access is not set as a sender", "AccessType");
            }

            using (var publisher = new ZmqPublishSocket()
                {
                    Identity = Guid.NewGuid().ToByteArray(),
                    RecoverySeconds = 10
                })
            {
                publisher.Bind(this.host);
                Thread.Sleep(500);
                publisher.Send(message);
            }
        }
        
        public string ReadNext()
        {
            if (this.accessType != AccessType.Receiver)
            {
                throw new InvalidPropertyException("Queue access is not set as a receiver", "AccessType");
            }

            string result = null;
            subscriber.Receive(out result, true);            
            return result;
        }

        public void Dispose()
        {
            if (subscriber != null)
            {
                subscriber.Dispose();
            }
        }
    }
}